"""Main module."""
import io
import os
import zipfile
import requests
import pandas as pd

from . config import (
    GN_DL_URL,
    GN_PL_HEADERS,
    FEATURE_CODE_HEADERS,
    FEATURE_CODE_LANG
)


def clean_feature_df(df):
    """ replace empty description cells and remove nan-rows

    :param df: pandas.Dataframe derived from geonames features
    :param type: pandas.DataFrame

    :return: a cleaned dataframe
    :rtype: pandas.DataFrame
    """
    df['description'] = df['description'].fillna(df['pref_label'])
    df['group'] = df['code'].str[0]
    return df.dropna()


def load_feature_codes():
    """
    load package copy of feature codes into dataframe

    :return: A pandas.DataFrame with the feature codes
    :rtype: pandas.DataFrame
    """
    df = pd.read_csv('fixtures/features_en.csv')
    return clean_feature_df(df)


def dl_feature_codes(lang="en"):
    """
    downloads geonames feature_codes

    :param lang: The language of the feature codes
    :param type: str

    :return: geonames feature codes as string
    :rtype: str
    """
    if lang in FEATURE_CODE_LANG:
        url = f"{GN_DL_URL}featureCodes_{lang}.txt"
        r = requests.get(url)
        if r.status_code == 200:
            return r.text
        else:
            print(f"something weng wrong, status code: {r.status_code}")
            return ""
    else:
        print(f"feature codes not available in selected lang: {lang}")
        return ""


def feature_codes_df(lang="en"):
    """
    downloads geonames feature_codes and returns a pandas.DataFrame

    :param lang: The language of the feature codes
    :param type: str

    :return: geonames feature codes as pandas.DataFrame
    :rtype: pandas.DataFrame
    """
    ft_codes = dl_feature_codes(lang)
    if ft_codes:
        data = io.StringIO(ft_codes)
        df = pd.read_csv(data, sep='\t', names=FEATURE_CODE_HEADERS)
        return clean_feature_df(df)
    else:
        return None


def download_country_zip(country_code, out_dir='temp'):
    """
    downloads a geonames country zip like e.g.\
        http://download.geonames.org/export/dump/AT.zip \
            and returns the location of the zipped file

    :param country_code: The country code of the country to download e.g. AT
    :type country_code: str

    :param out_dir: a directory path to store the downloaded file
    :type out_dir: str

    :return: The path of the downloaded zip
    :rtype: str
    """
    url = f"{GN_DL_URL}{country_code}.zip"
    file_name = f"{country_code}.zip"
    save_path = f"{os.path.join(out_dir, file_name)}"
    os.makedirs(out_dir, exist_ok=True)
    r = requests.get(url, stream=True)
    if r.status_code == 200:
        with open(save_path, 'wb') as fd:
            for chunk in r.iter_content(chunk_size=128):
                fd.write(chunk)
        return save_path
    else:
        return ""


def unzip_country_zip(zipped_file):
    """
    unzipps a geonames country zip like e.g.\
        temp/AT.zip and returns the filename of the unzipped file

    :param country_code: The location of the zipped file e.g. temp/AT.zip
    :type country_code: str

    :return: The unzipped file e.g. temp/AT.txt
    :rtype: str
    """
    if zipped_file:
        path_name, file_name = os.path.split(zipped_file)
        with zipfile.ZipFile(zipped_file, "r") as zip_ref:
            zip_ref.extractall(path_name)
            unzipped_file = zipped_file.replace('.zip', '.txt')
    else:
        unzipped_file = ""
    return unzipped_file


def download_and_unzip_country_zip(country_code, out_dir='temp'):
    """
    downloads and unzipps a geonames country zip like e.g.\
        http://download.geonames.org/export/dump/AT.zip \
            and returns the filename of the unzipped file

    :param country_code: The country code of the country to download e.g. AT
    :type country_code: str

    :param out_dir: a directory path to store the downloaded file
    :type out_dir: str

    :return: The path of the downloaded and extracted zip
    :rtype: str
    """

    zipped_file = download_country_zip(country_code, out_dir=out_dir)
    if zipped_file:
        unzipped = unzip_country_zip(zipped_file)
    else:
        unzipped = ""
    return unzipped


def countries_as_df(input_file):
    """
    returns a geonames-download file as pandas Dataframe objects

    :param input_file: The location of an unzipped geonames\
        file e.g. temp/AT.txt
    :type input_file: str

    :return: a pandas.Dataframe objects
    :rtype: pandas.Dataframe
    """

    df = pd.read_csv(input_file, sep='\t', names=GN_PL_HEADERS)
    return df


def download_to_df(country_code, out_dir='temp'):
    """
    downloads and unzipps a geonames country zip like e.g.\
        http://download.geonames.org/export/dump/AT.zip \
            and returns the data as pandas.DataFrame

    :param country_code: The country code of the country to download e.g. AT
    :type country_code: str

    :param out_dir: a directory path to store the downloaded file
    :type out_dir: str

    :return: The data as pandas.DataFrame
    :rtype: `pandas.DataFrame`
    """

    input_file = download_and_unzip_country_zip(country_code, out_dir=out_dir)
    if input_file:
        df = countries_as_df(input_file)
    else:
        df = None
    return df



#!/usr/bin/env python

"""Tests for `acdh_geonames_utils` package."""

import os
import unittest
from click.testing import CliRunner

from acdh_geonames_utils import acdh_geonames_utils as gn
from acdh_geonames_utils import cli

good_country_code = 'YU'
bad_country_code = 'BAAAD'

good_ft_code = "en"
bad_ft_code = "de"

TEST_GN_FILE = os.path.join(
    "./fixtures",
    "AL.txt"
)


class TestAcdh_geonames_utils(unittest.TestCase):
    """Tests for `acdh_geonames_utils` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_001_download(self):
        """Test download of zip."""
        good = gn.download_country_zip(good_country_code)
        bad = gn.download_country_zip(bad_country_code)
        self.assertTrue(good.endswith(f"{good_country_code}.zip"))
        self.assertEqual(bad, "")

    def test_002_download_and_unzip(self):
        """Test download and unzip."""
        good = gn.download_and_unzip_country_zip(good_country_code)
        bad = gn.download_and_unzip_country_zip(bad_country_code)
        self.assertTrue(good.endswith(f"{good_country_code}.txt"))
        self.assertEqual(bad, "")

    def test_003_unzip(self):
        """Test unzipping of zip."""
        bad = gn.unzip_country_zip("")
        self.assertEqual(bad, "")

    def test_004_file_to_df(self):
        """Test loading file into pandas.DataFrame"""
        df = gn.countries_as_df(TEST_GN_FILE)
        self.assertEqual(len(df), 9356)

    def test_005_dl_to_df(self):
        """Test loading download into pandas.DataFrame"""
        good_df = gn.download_to_df('YU')
        bad_df = gn.download_to_df('YUUU')
        self.assertEqual(len(good_df), 1)
        self.assertFalse(bad_df)

    def test_006_dl_ft(self):
        good = gn.dl_feature_codes(good_ft_code)
        bad = gn.dl_feature_codes(bad_ft_code)
        self.assertTrue(good != "")
        self.assertTrue(bad == "")

    def test_007_dl_ft_as_df(self):
        good = gn.feature_codes_df(good_ft_code)
        bad = gn.feature_codes_df(bad_ft_code)
        self.assertIsNotNone(good)
        self.assertIsNone(bad)

    def test_command_line_interface(self):
        """Test the CLI."""
        runner = CliRunner()
        result = runner.invoke(cli.main)
        assert result.exit_code == 0
        assert 'acdh_geonames_utils.cli.main' in result.output
        help_result = runner.invoke(cli.main, ['--help'])
        assert help_result.exit_code == 0
        assert '--help  Show this message and exit.' in help_result.output



GN_DL_URL = "http://download.geonames.org/export/dump/"

GN_PL_HEADERS = [
    "geonameid",
    "name",
    "asciiname",
    "alternatenames",
    "latitude",
    "longitude",
    "feature class",
    "feature code",
    "country code",
    "cc2",
    "admin1 code",
    "admin2 code",
    "admin3 code",
    "admin4 code",
    "population",
    "elevation",
    "dem",
    "timezone",
    "modification date"
]

FEATURE_CODE_LANG = [
    "bg",
    "en",
    "nb",
    "nn",
    "no",
    "ru",
    "sv",
]

FEATURE_CODE_HEADERS = [
    "code",
    "pref_label",
    "description"
]



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'Click>=7.0',
    'requests',
    'pandas>=1.1',
]

setup_requirements = []

test_requirements = []

setup(
    author="Peter Andorfer",
    author_email='peter.andorfer@oeaw.ac.at',
    python_requires='>=3.8',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
    ],
    description="Utility functions to interact with geonames.org",
    entry_points={
        'console_scripts': [
            'acdh_geonames_utils=acdh_geonames_utils.cli:main',
        ],
    },
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    package_data={
        '': ['fixtures/*.*'],
    },
    keywords='acdh_geonames_utils',
    name='acdh_geonames_utils',
    packages=find_packages(include=[
        'acdh_geonames_utils',
        'acdh_geonames_utils.*']
    ),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/acdh-oeaw/acdh_geonames_utils',
    version='0.4.0',
    zip_safe=False,
)
